import pprint

from ninja import NinjaAPI

from corptools.task_helpers.char_tasks import get_token


class TestingApiEndpoints:
    tags = ["Extras"]

    def __init__(self, api: NinjaAPI):
        @api.get(
            "/extras/test/newapi1",
            tags=["Testing"]
        )
        def get_test_api_bravadosih(request):
            """
                Not for real use!
                this is kinda like bravado in use... no type hinting
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}

            from esi.clients import OpenAPIEsiClientProvider
            api = OpenAPIEsiClientProvider()

            return api.client.Alliance.get_alliances_alliance_id(alliance_id=1900696668)

        @api.get(
            "/extras/test/newapi2",
            tags=["Testing"]
        )
        def get_test_api_openapi(request):
            """
                Not for real use!
                this has type hinting
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}

            from esi.clients import OpenAPIEsiClientProvider
            api = OpenAPIEsiClientProvider()

            from esi.client.api import AllianceApi

            return AllianceApi(api.client).get_alliances_alliance_id(alliance_id=1900696668)

        @api.get(
            "/extras/test/newapi3",
            tags=["Testing"]
        )
        def get_test_api_openapi_3(request, name=""):
            """
                Not for real use!
                this has type hinting
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}

            from corptools.models import EveItemType
            item, created = EveItemType.objects.get_or_create_from_esi_name(
                name)
            return {
                "created": created,
                "item_name": item.name,
                "item_id": item.type_id
            }

        @api.get(
            "/extras/test/newapi4",
            tags=["Testing"]
        )
        def get_test_api_openapi_4(request, character_id=0):
            """
                Not for real use!
                this has type hinting
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}

            from corptools.task_helpers.char_tasks import (
                update_character_assets,
            )

            update_character_assets(character_id, force_refresh=True)
            return "done"

        @api.get(
            "/extras/test/newapi5",
            tags=["Testing"]
        )
        def get_test_api_oiaopenapi3(request):
            """
                Not for real use!
                this has type hinting
            """
            if not request.user.is_superuser:
                return 403, {"message": "Hard no pall!"}
            from aiopenapi3 import OpenAPI

            from esi.clients import EsiClientProvider
            esi2 = EsiClientProvider()
            res = esi2.client.Sovereignty.get_sovereignty_campaigns()
            res.request_config.also_return_response = True
            result, headers = res.result()
            etag1 = headers.headers["etag"]
            print("here")
            # res = esi.client.Alliance.GetAlliances(
            # )
            # result, headers = res.result(return_response=True)
            # # print([result, headers])
            # res = esi.client.Alliance.GetAlliancesAllianceId(
            #     alliance_id=1900696668
            # )
            # result, headers = res.result(return_response=True)
            # # print([result, headers])
            # res = esi.client.Status.GetStatus(
            # )
            # result, headers = res.result(return_response=True)
            # # print([result, headers])
            # res = esi.client.Incursions.GetIncursions(
            # )
            # result, headers = res.result(return_response=True)
            # # print([result, headers])
            from esi import openapi_clients
            from esi.models import Token
            from esi.rate_limiting import ESIRateLimitBucket

            from corptools.models import CharacterAsset, MapSystem
            esi = openapi_clients.ESIClientProvider(
                compatibility_date="2025-08-26",
                ua_appname="ack-testing-django-esi",
                ua_version="0.0.1a1"
            )

            # def hit_api():
            #     esi.client.Alliance.GetAlliances(
            # ).result()

            # for i in range(0,50):
            #     print(f"Hit {i}")
            #     hit_api()
            # req_scopes = ['esi-assets.read_assets.v1']
            req_scopes = ['esi-fleets.write_fleet.v1']
            cid = 2119950231
            # cid = 755166922
            # token = get_token(cid, req_scopes)

            # try:
            #     esi.client.Alliance.GetAlliances(
            #         token=token
            #     ).result()
            # except ValueError:
            #     print("This is good")

            # res = esi.client.Assets.GetCharactersCharacterIdAssets(
            #     character_id=755166922,
            #     token=token
            # )

            # result, headers = res.results(return_response=True)

            # print(result[0])
            # for a in result:
            #     print(a.type_id)
            # flt = esi.client.Fleets.GetCharactersCharacterIdFleet(
            #     body={"is_free_move": True, "motd": "Testing 1, 2, 3"},
            #     character_id=cid,
            #     token=token
            # ).result()

            # flt_up = esi.client.Fleets.PutFleetsFleetId(
            #     body={"is_free_move": True, "motd": "Testing 1, 2, 3"},
            #     fleet_id=1234,
            #     token=token
            # )
            # flt_up.result()

            # print(flt)
            # print(flt_up)
            # res = esi.client.Assets.GetCharactersCharacterIdAssets(
            #     character_id=755166922,
            #     token=Token.objects.all().require_scopes_exact("publicData").first()
            # )

            # result, headers = res.results(return_response=True)
            # print([len(result), headers])

            # from esi import clients as clients2
            # esi2 = clients2.EsiClientProvider(
            #     ua_appname = "ack-testing-django-esi",
            #     ua_version = "0.0.1a1"
            # )
            # token = get_token(755166922, req_scopes)
            # res = esi2.client.Assets.get_characters_character_id_assets(
            #     character_id=755166922,
            #     token=token.valid_access_token()
            # )
            print(etag1)

            from esi import openapi_clients
            esi = openapi_clients.ESIClientProvider(
                compatibility_date="2025-08-26",
                ua_appname="ack-testing-django-esi",
                ua_version="0.0.1a1"
            )

            req = esi.client.Sovereignty.GetSovereigntyCampaigns()

            resp2, result2 = req.result(
                return_response=True
            )
            # result2 = res.results()
            etag2 = result2.headers["etag"]
            # print(etag2)

            resp3, result3 = esi.client.Sovereignty.GetSovereigntyCampaigns().result(
                return_response=True,
                use_cache=False
            )
            etag3 = result3.headers["etag"]
            # print(etag3)

            return [etag2, str(resp2), etag3, str(resp3)]



from model_utils import Choices

from django.db import models

from .audits import CharacterAudit, EveLocation
from .eve_models import EveItemType, MapRegion
from .wallets import CorporationWalletDivision


class MarketOrder(models.Model):
    order_id = models.BigIntegerField(primary_key=True)

    duration = models.IntegerField()
    escrow = models.DecimalField(
        max_digits=20, decimal_places=2, null=True, default=None)
    is_buy_order = models.BooleanField(null=True, default=None)
    issued = models.DateTimeField()
    location_id = models.BigIntegerField()
    location_name = models.ForeignKey(
        EveLocation, on_delete=models.SET_NULL, null=True, default=None)

    min_volume = models.IntegerField(null=True, default=None)
    price = models.DecimalField(
        max_digits=20, decimal_places=2, null=True, default=None)
    _range_enum = Choices('1', '10', '2', '20', '3', '30',
                          '4', '40', '5', 'region', 'solarsystem', 'station')
    order_range = models.CharField(max_length=30, choices=_range_enum)
    region_id = models.IntegerField()
    region_name = models.ForeignKey(
        MapRegion, on_delete=models.SET_NULL, null=True, default=None)
    type_id = models.IntegerField()
    type_name = models.ForeignKey(
        EveItemType, on_delete=models.SET_NULL, null=True, default=None)
    volume_remain = models.IntegerField()
    volume_total = models.IntegerField()

    _state_enum = Choices('cancelled', 'expired', 'active')
    state = models.CharField(max_length=30, choices=_state_enum)

    class Meta:
        abstract = True


class CorporationMarketOrder(MarketOrder):
    wallet_division = models.ForeignKey(
        CorporationWalletDivision, on_delete=models.CASCADE)
    issued_by = models.IntegerField()


class CharacterMarketOrder(MarketOrder):
    character = models.ForeignKey(CharacterAudit, on_delete=models.CASCADE)
    is_corporation = models.BooleanField()



import datetime
from typing import ClassVar

from django.db import models
from django.utils import timezone
from django.utils.translation import gettext_lazy as _, ngettext_lazy

from allianceauth.eveonline.evelinks import eveimageserver

from ..managers import (
    EveCategoryManager, EveGroupManager, EveItemTypeManager, EveMoonManager,
    EveNameManager, EvePlanetManager,
)


class EveName(models.Model):
    objects: ClassVar[EveNameManager] = EveNameManager()

    eve_id = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=255)
    category = models.CharField(max_length=50)

    # optionals for character/corp
    corporation = models.ForeignKey(
        'EveName', on_delete=models.SET_NULL, null=True, default=None, related_name="corp")
    alliance = models.ForeignKey(
        'EveName', on_delete=models.SET_NULL, null=True, default=None, related_name="alli")
    last_update = models.DateTimeField(auto_now=True)

    CHARACTER = "character"
    CORPORATION = "corporation"
    ALLIANCE = "alliance"

    def __str__(self):
        return self.name

    def get_image_url(self):
        if self.category == self.CHARACTER:
            return eveimageserver.character_portrait_url(self.eve_id)
        elif self.category == self.CORPORATION:
            return eveimageserver.corporation_logo_url(self.eve_id)
        elif self.category == self.ALLIANCE:
            return eveimageserver.alliance_logo_url(self.eve_id)
        elif self.category == 'faction':  # CCP...
            return eveimageserver.corporation_logo_url(self.eve_id)

    def needs_update(self):
        return self.last_update + datetime.timedelta(days=30) < timezone.now()


class EveItemCategory(models.Model):
    objects: ClassVar[EveCategoryManager] = EveCategoryManager()
    category_id = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=255)  # unknown max

    def __str__(self):
        return self.name


class EveItemGroup(models.Model):
    objects: ClassVar[EveGroupManager] = EveGroupManager()
    group_id = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=255)  # unknown max
    category = models.ForeignKey(
        EveItemCategory, on_delete=models.SET_NULL, null=True, default=None)

    def __str__(self):
        return self.name


class EveItemType(models.Model):
    objects: ClassVar[EveItemTypeManager] = EveItemTypeManager()
    type_id = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=255)  # unknown max
    group = models.ForeignKey(
        EveItemGroup, on_delete=models.SET_NULL, null=True, default=None)
    description = models.TextField(null=True, default=None)
    mass = models.FloatField(null=True, default=None)
    packaged_volume = models.FloatField(null=True, default=None)
    portion_size = models.FloatField(null=True, default=None)
    volume = models.FloatField(null=True, default=None)
    published = models.BooleanField()
    radius = models.FloatField(null=True, default=None)

    def __str__(self):
        return self.name


class EveItemDogmaAttribute(models.Model):
    eve_type = models.ForeignKey(
        EveItemType, on_delete=models.SET_NULL, null=True, default=None)
    attribute_id = models.BigIntegerField(null=True, default=None)
    value = models.FloatField(null=True, default=None)


class InvTypeMaterials(models.Model):
    qty = models.IntegerField()
    eve_type = models.ForeignKey(
        EveItemType, on_delete=models.SET_NULL, null=True, default=None)
    type_id = models.IntegerField()
    material_type_id = models.IntegerField()
    met_type = models.ForeignKey(
        EveItemType, on_delete=models.SET_NULL, null=True, default=None, related_name="met_type")


class MapRegion(models.Model):
    region_id = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=255)
    description = models.TextField(null=True, default=None, )

    def __str__(self):
        return self.name


class MapConstellation(models.Model):
    constellation_id = models.BigIntegerField(primary_key=True)
    name = models.CharField(max_length=255)
    region = models.ForeignKey(
        MapRegion, on_delete=models.SET_NULL, null=True, default=None)

    def __str__(self):
        return self.name


class MapSystem(models.Model):
    system_id = models.BigIntegerField(primary_key=True)
    security_status = models.FloatField()
    name = models.CharField(max_length=255)
    x = models.FloatField()
    y = models.FloatField()
    z = models.FloatField()
    security_class = models.CharField(max_length=255, null=True, default=None)
    star_id = models.IntegerField(null=True, default=None)
    constellation = models.ForeignKey(
        MapConstellation, on_delete=models.SET_NULL, null=True, default=None)

    def __str__(self):
        return self.name


class MapSystemGate(models.Model):
    from_solar_system = models.ForeignKey(
        MapSystem, on_delete=models.CASCADE, related_name="from_system")
    to_solar_system = models.ForeignKey(
        MapSystem, on_delete=models.CASCADE, related_name="to_system")

    def __str__(self):
        return (self.from_solar_system_id, self.to_solar_system_id)


class MapSystemPlanet(models.Model):
    objects: ClassVar[EvePlanetManager] = EvePlanetManager()

    planet_id = models.IntegerField(primary_key=True)
    system = models.ForeignKey(
        MapSystem, on_delete=models.CASCADE, related_name="planet")
    name = models.CharField(max_length=255)

    x = models.FloatField()
    y = models.FloatField()
    z = models.FloatField()

    eve_type = models.ForeignKey(
        EveItemType, on_delete=models.SET_NULL, null=True, default=None)

    def __str__(self):
        return (self.name)


class MapSystemMoon(models.Model):

    objects: ClassVar[EveMoonManager] = EveMoonManager()

    moon_id = models.IntegerField(primary_key=True)
    system = models.ForeignKey(
        MapSystem, on_delete=models.CASCADE, related_name="moon")
    name = models.CharField(max_length=255)

    x = models.FloatField()
    y = models.FloatField()
    z = models.FloatField()

    def __str__(self):
        return (self.name)


class MapJumpBridge(models.Model):
    structure_id = models.BigIntegerField(primary_key=True)
    from_solar_system = models.ForeignKey(
        MapSystem, on_delete=models.CASCADE, related_name="bridge_from_system")
    to_solar_system = models.ForeignKey(
        MapSystem, on_delete=models.CASCADE, related_name="bridge_to_system")
    owner = models.ForeignKey(
        EveName, on_delete=models.SET_NULL, null=True, default=None)
    updated = models.DateTimeField(auto_now=True)
    manually_input = models.BooleanField(default=False)

    def __str__(self):
        return f"{self.from_solar_system.name} >> {self.to_solar_system.name} ({self.structure_id}) (Auto: {not self.manually_input})"


class TypePrice(models.Model):
    item = models.ForeignKey(EveItemType, on_delete=models.DO_NOTHING)
    price = models.DecimalField(max_digits=20, decimal_places=2)
    last_update = models.DateTimeField(auto_now=True)


# ************ is this needed any more?

class OreTax(models.Model):
    item = models.ForeignKey(EveItemType, on_delete=models.DO_NOTHING)
    price = models.DecimalField(max_digits=20, decimal_places=2)
    last_update = models.DateTimeField(auto_now=True)


class OreTaxRates(models.Model):
    tag = models.CharField(max_length=500, default="Mining Tax")
    refine_rate = models.DecimalField(
        max_digits=5, decimal_places=2, default=87.5)
    ore_rate = models.DecimalField(max_digits=5, decimal_places=2)  # normal
    ubiquitous_rate = models.DecimalField(
        max_digits=5, decimal_places=2)  # ubiq
    common_rate = models.DecimalField(max_digits=5, decimal_places=2)  # comon
    uncommon_rate = models.DecimalField(
        max_digits=5, decimal_places=2)  # uncom
    rare_rate = models.DecimalField(max_digits=5, decimal_places=2)  # rare
    excptional_rate = models.DecimalField(
        max_digits=5, decimal_places=2)  # best
